import * as React from "react";
import FormItemCommon from "./form-item";
import { updateActiveSection } from "./form-section.props";
import { FormItemArrayClassNameContract } from "../class-name-contracts/";
import { ManagedJSSProps } from "@microsoft/fast-jss-manager-react";
import { ManagedClasses } from "@microsoft/fast-components-class-name-contracts-base";
import FormItemBase from "./form-item.base";
export declare enum ItemConstraints {
    minItems = "minItems",
    maxItems = "maxItems"
}
export declare enum ArrayAction {
    add = "add",
    remove = "remove"
}
export interface FormItemArrayProps extends FormItemCommon {
    /**
     * The schema
     */
    schema: any;
    /**
     * The location of the schema
     */
    schemaLocation: string;
    /**
     * The location of the data
     */
    dataLocation: string;
    /**
     * The string to use for an untitled schema
     */
    untitled: string;
    /**
     * The callback to update a different active section and/or component
     */
    onUpdateActiveSection: updateActiveSection;
}
/**
 * Schema form component definition
 * @extends React.Component
 */
declare class FormItemArray extends FormItemBase<FormItemArrayProps & ManagedClasses<FormItemArrayClassNameContract>, {}> {
    static displayName: string;
    render(): React.ReactNode;
    private renderLabel;
    /**
     * Render a button for adding an item to the array
     */
    private renderAddArrayItemTrigger;
    /**
     * Render a button for removing an item from the array
     */
    private renderDeleteArrayItemTrigger;
    /**
     * Array add/remove item click handler factory
     */
    private arrayItemClickHandlerFactory;
    /**
     * Array section link click handler factory
     */
    private arrayClickHandlerFactory;
    /**
     * Handles adding an array item
     */
    private handleAddArrayItem;
    /**
     * Handles removing an array item
     */
    private handleRemoveArrayItem;
    /**
     * Generates UI representing an item of an array
     */
    private generateArrayLinkItem;
    /**
     * Generates UI for all items in an array
     */
    private generateArrayLinkItems;
    /**
     * Handle user drag and drop interactions
     */
    private handleSort;
    /**
     * Generates the links to an array section to be activated
     */
    private renderExistingArrayItems;
}
export { FormItemArray };
declare const _default: React.ComponentClass<ManagedJSSProps<FormItemArrayProps & FormItemCommon, FormItemArrayClassNameContract, {}>, any>;
export default _default;
