import * as React from "react";
import { getStringValue } from "./form-item.utilities";
import styles from "./form-item.number-field.style";
import manageJss from "@microsoft/fast-jss-manager-react";
import FormItemBase from "./form-item.base";
/**
 * Schema form component definition
 * @extends React.Component
 */
class FormItemNumberField extends FormItemBase {
    constructor() {
        super(...arguments);
        /**
         * Handles the onChange event for the input
         */
        this.handleChange = ({ target: { value } }) => {
            const normalizedValue = this.getNumberValue(value);
            return this.props.onChange(this.props.dataLocation, normalizedValue);
        };
    }
    render() {
        const value = getStringValue(this.props.data, this.props.default);
        return (React.createElement("div", { className: this.props.managedClasses.formItemNumberField },
            React.createElement("div", { className: this.props.managedClasses.formItemNumberField_control },
                React.createElement("label", { className: this.props.managedClasses.formItemNumberField_controlLabel, htmlFor: this.props.dataLocation }, this.props.label),
                React.createElement("input", { className: this.props.managedClasses.formItemNumberField_controlInput, id: this.props.dataLocation, type: "number", value: value, name: `number${value}`, onChange: this.handleChange, min: this.props.min, max: this.props.max, step: this.props.step, disabled: this.props.disabled })),
            React.createElement("div", { className: this.props.managedClasses.formItemNumberField_softRemove }, this.renderSoftRemove(this.props.managedClasses.formItemNumberField_softRemoveInput))));
    }
    /**
     * Make sure this is not NaN
     */
    getNumberValue(value) {
        const newValue = parseFloat(value);
        if (!isNaN(newValue)) {
            return newValue;
        }
        return this.props.data;
    }
}
FormItemNumberField.displayName = "FormItemNumberField";
export default manageJss(styles)(FormItemNumberField);
