import * as React from "react";
import styles from "./form-item.select.style";
import manageJss from "@microsoft/fast-jss-manager-react";
import FormItemBase from "./form-item.base";
/**
 * Schema form component definition
 * @extends React.Component
 */
/* tslint:disable-next-line */
class FormItemSelect extends FormItemBase {
    constructor() {
        super(...arguments);
        /**
         * Handles the onChange of the select element
         */
        this.handleChange = (event) => {
            this.props.onChange(this.props.dataLocation, this.parse(event.target.value));
        };
    }
    /**
     * Renders the component
     */
    render() {
        if (this.props.options.length === 1 && this.props.required) {
            return null;
        }
        const value = typeof this.props.data !== "undefined"
            ? this.props.data
            : this.props.default || this.props.options[0];
        return (React.createElement("div", { className: this.props.managedClasses.formItemSelect },
            React.createElement("div", { className: this.props.managedClasses.formItemSelect_control },
                React.createElement("label", { className: this.props.managedClasses.formItemSelect_controlLabel }, this.props.label),
                React.createElement("span", { className: this.generateControlSpanClassNames() },
                    React.createElement("select", { className: this.props.managedClasses.formItemSelect_controlInput, onChange: this.handleChange, value: JSON.stringify(value), disabled: this.props.disabled }, this.renderOptions()))),
            React.createElement("div", { className: this.props.managedClasses.formItemSelect_softRemove }, this.renderSoftRemove(this.props.managedClasses.formItemSelect_softRemoveInput))));
    }
    /**
     * Generates class names
     */
    generateControlSpanClassNames() {
        let classNames = this.props.managedClasses.formItemSelect_controlSpan;
        if (this.props.disabled) {
            classNames = `${classNames} ${this.props.managedClasses.formItemSelect_controlSpan__disabled}`;
        }
        return classNames;
    }
    /**
     * Stringify the select value
     */
    stringify(value) {
        try {
            return JSON.stringify(value);
        }
        catch (e) {
            return value;
        }
    }
    /**
     * Parse the select value
     */
    parse(value) {
        try {
            return JSON.parse(value);
        }
        catch (e) {
            return value;
        }
    }
    /**
     * Renders the selects option elements
     */
    renderOptions() {
        return this.props.options.map((item, index) => {
            const stringifiedItem = this.stringify(item);
            return (React.createElement("option", { key: index, value: stringifiedItem }, typeof item === "string" || typeof item === "number"
                ? item
                : stringifiedItem));
        });
    }
}
FormItemSelect.displayName = "FormItemSelect";
export default manageJss(styles)(FormItemSelect);
