import { getLocationsFromObject } from "@microsoft/fast-data-utilities-react";
import { cloneDeep, get, set, unset } from "lodash-es";
const pluginIdKeyword = "formPluginId";
/**
 *
 */
export function mapPluginsToSchema(schema, data, plugins = []) {
    const mappedSchema = cloneDeep(schema);
    // identify locations of the plugins
    const pluginModifiedSchemaLocations = getSchemaLocationsOfPlugins(schema);
    // going from the longest length to the shortest, resolve the data with plugins
    pluginModifiedSchemaLocations.forEach((pluginModifiedSchemaLocation) => {
        const pluginId = get(pluginModifiedSchemaLocation.schema, `${pluginModifiedSchemaLocation.schemaLocation}.${pluginIdKeyword}`);
        const pluginResolver = plugins.find((plugin) => {
            return plugin.matches(pluginId);
        });
        const pluginSchema = get(schema, pluginModifiedSchemaLocation.schemaLocation);
        if (pluginResolver !== undefined) {
            const pluginResolverMapping = getPluginResolverSchemaMap({
                pluginSchema,
                pluginData: data,
                pluginResolver,
                schemaLocation: pluginModifiedSchemaLocation.schemaLocation,
            });
            pluginResolverMapping.forEach((pluginResolverMappingItem) => {
                if (typeof pluginResolverMappingItem.schema === "undefined") {
                    unset(mappedSchema, pluginResolverMappingItem.schemaLocation);
                }
                else {
                    set(mappedSchema, pluginResolverMappingItem.schemaLocation, pluginResolverMappingItem.schema);
                }
            });
        }
    });
    return mappedSchema;
}
/**
 * Finds the schema locations of types mapped to plugins
 */
export function getSchemaLocationsOfPlugins(schema, schemaLocationPrefix = "") {
    const schemaLocationsOfPlugins = [];
    // get all schema locations
    const schemaLocations = getLocationsFromObject(schema).concat([""]);
    // determine if the schema location is mapped to a plugin
    schemaLocations.forEach((schemaLocation) => {
        const subSchema = schemaLocation === "" ? schema : get(schema, schemaLocation);
        const schemaLocationOfPlugin = schemaLocationPrefix === ""
            ? schemaLocation
            : `${schemaLocationPrefix}.${schemaLocation}`;
        // check to see if the schema location matches with the current schema and includes a plugin identifier
        if (typeof get(subSchema, pluginIdKeyword) === "string" &&
            schemaLocationsOfPlugins.findIndex(pluginPartialFindIndexCallback(schemaLocationOfPlugin)) === -1) {
            schemaLocationsOfPlugins.push({
                schema,
                schemaLocation: schemaLocationOfPlugin,
            });
        }
    });
    return schemaLocationsOfPlugins;
}
/**
 * Callback to determine if a string is found within an array of plugin locations
 */
function pluginPartialFindIndexCallback(schemaLocation) {
    return (formPluginLocation) => {
        return (schemaLocation.slice(0, formPluginLocation.schemaLocation.length) ===
            formPluginLocation.schemaLocation);
    };
}
/**
 * Get the resolved data from a plugin and the schema location to map to
 */
function getPluginResolverSchemaMap(pluginResolverSchemaMapConfig) {
    const pluginResolverMapping = [];
    const pluginResolverMappingItem = {
        schemaLocation: pluginResolverSchemaMapConfig.schemaLocation,
        schema: pluginResolverSchemaMapConfig.pluginResolver.resolver(pluginResolverSchemaMapConfig.pluginSchema, pluginResolverSchemaMapConfig.pluginData),
    };
    pluginResolverMapping.push(pluginResolverMappingItem);
    return pluginResolverMapping;
}
