import * as React from "react";
import { BreadcrumbItemEventHandler, FormChildOptionItem, FormState } from "./form.props";
export declare enum PropertyKeyword {
    properties = "properties",
    reactProperties = "reactProperties"
}
export interface NavigationItem {
    dataLocation: string;
    schemaLocation: string;
    title: string;
    data: any;
    schema: any;
}
export interface NavigationItem {
    dataLocation: string;
    data: any;
    schema: any;
}
export interface BreadcrumbItem {
    href: string;
    text: string;
    onClick: (e: React.MouseEvent<HTMLAnchorElement>) => void;
}
export declare type HandleBreadcrumbClick = (schemaLocation: string, dataLocation: string, schema: any) => BreadcrumbItemEventHandler;
/**
 * Gets the data cache based on a new data object and
 * previous data object
 */
export declare function getDataCache(dataCache: any, newData: any): any;
/**
 * Determines the navigation from
 * - section links
 * - child components
 * - array items
 * - breadcrumb links
 */
export declare function getActiveComponentAndSection(schemaLocation: string, dataLocation: string, schema?: any): Partial<FormState>;
/**
 * Gets locations from individual location segments
 * Example:
 * getLocationsFromSegments(["children[0].props.object"])
 * output: ["children[0]", "children[0].props", "children[0].props.object"]
 */
export declare function getLocationsFromSegments(segments: string[]): string[];
/**
 * Gets the navigational items
 */
export declare function getNavigation(dataLocation: string, data: any, schema: any, childOptions: FormChildOptionItem[]): NavigationItem[];
/**
 * Get a single navigation item
 */
export declare function getNavigationItem(dataLocation: string, schemaLocation: string, schema: any, data: any): NavigationItem;
/**
 * Get React's default children
 */
export declare function getReactDefaultChildren(): FormChildOptionItem[];
/**
 * Gets the data location from the current component
 */
export declare function getCurrentComponentDataLocation(dataLocation: string, lastComponentDataLocation: string): string;
/**
 * Removes any references to array index
 */
export declare function normalizeSchemaLocation(schemaLocation: string): string;
/**
 * Check to see if we are on the root location
 */
export declare function isRootLocation(location: string): boolean;
/**
 * Gets breadcrumbs from navigation items
 */
export declare function getBreadcrumbs(navigation: NavigationItem[], handleClick: HandleBreadcrumbClick): BreadcrumbItem[];
/**
 * Finds the schema using the data location
 */
export declare function getSchemaByDataLocation(currentSchema: any, data: any, dataLocation: string, childOptions: FormChildOptionItem[]): any;
/**
 * Finds the component using the schema id
 */
export declare function getComponentByDataLocation(id: string, childOptions: FormChildOptionItem[]): any;
